<?php
// Créateur : Mathieu Chartier
// Date : 21/10/2016
// Version : 1.1

// Fonction de génération d'un texte autour d'un mot précis
// Paramètre 1 : $texte correspond au texte dans lequel rechercher et générer un extrait
// Paramètre 2 : $mot correspond au mot autour duquel va être dessiné l'extrait
// Paramètre 3 : $limite est le nombre de mot à récupérer autour du mot recherché
// Paramètre 4 : $numOccurrence équivaut au numéro de l'occurrence du mot trouvé dans le texte ("5" étant la 5e présence du mot dans le texte, pour générer l'extrait plus loin dans le texte)
// Paramètre 5 : $chainePrec correspond au texte à écrire avant l'extrait
// Paramètre 6 : $chaineSuiv correspond au texte à écrire après l'extrait
function generateWrapText($texte = '', $mot = '', $limite = 15, $numOccurrence = 5, $strong = true, $chainePrec = "(...)", $chaineSuiv = "...") {
	// Dans le cas où malgré la suppression des accents $mot reste un tableau
	if(is_array($mot)) {
		$indice = rand(0, count($mot)); // On prend un des mots au hasard
		$mot = $mot[$indice];
	}
	
	// Coupe la chaîne mot à mot
	$chaineTexte = mb_split("([[:space:]]|[\(\)\[\]\{\},;:!?<>])+", strip_tags(trim($texte)));
	
	// Nettoyage de la chaine (optionnel)
	foreach($chaineTexte as $cle => $motUnique) {
		$cleanWord = trim($motUnique);
		$motUnique = preg_replace("#(\\n|\\r|&nbsp;)#i", " ", $motUnique);
		
		// Tableau nettoyé reconstitué
		$chaineTab[] = $motUnique;
	}
	
	// Récupère la clé du mot recherché
	$cleMot = array_keys($chaineTab, $mot); // Toutes les occurrences
	
	// On compte le nombre d'occurrences pour assurer de générer un snippet autour du mot cherché
	$nbOcc = count($cleMot, 1);

	if($numOccurrence > $nbOcc) {
		$numOccurrence = $nbOcc - 1; // On prend l'avant-dernière occurrence par exemple
	}
	
	// Paramètres par défaut
	$debutIncomplet = false;
	$finIncomplete = false;
	$chaineCreee = "";
	$chaine = "";

	// Extraction des mots suivants
	if(isset($cleMot[$numOccurrence])) {
		foreach($chaineTab as $cle => $valeur) {	
			if($cle == $cleMot[$numOccurrence]) {
				// "n" Mots précédents
				for($i = $limite; $i >= 1; $i--) {
					$posMotPrecedent = $cleMot[$numOccurrence]-$i;
					if(isset($chaineTab[$posMotPrecedent])) {
						$chaineCreee.= " ".$chaineTab[$posMotPrecedent];
					} else {
						$debutIncomplet = true;
					}
				}

				// Mot recherché (affiché en gras ou non)
				if($strong == true) {
					$chaineCreee.= " <strong>".$chaineTab[$cleMot[$numOccurrence]]."</strong>";
				} else {
					$chaineCreee.= " ".$chaineTab[$cleMot[$numOccurrence]];
				}
				
				// "n" Mots suivants
				for($i = 1; $i <= $limite; $i++) {
					$posMotSuivant = $cleMot[$numOccurrence]+$i;
					if(isset($chaineTab[$posMotSuivant])) {
						$chaineCreee.= " ".$chaineTab[$posMotSuivant];
					} else {
						$finIncomplete = true;
					}
				}
			}
		}

		// Ajout des caractères de début (... par exemple) si le découpage n'atteint pas le début du texte.
		if($debutIncomplet == false) {
			$chaine.= $chainePrec;
		}
		$chaine.= $chaineCreee; // Ajout de la chaîne reconstituée
		// Ajout des caractères de fin (... par exemple) si la phrase n'est pas au bout.
		if($finIncomplete == false) {
			$chaine.= $chaineSuiv;
		}
	}
	
	// Retourne le résultat
	return $chaine;
}

// Exemple d'utilisation
$phrase = "Ceci est une phrase écrite totalement au hasard pour donner un exemple d'utilisation du code.";
$motCible = "hasard";

echo generateWrapText($phrase, $motCible, 10);
?>